#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2010, 2011, 2013, 2014, 2016.

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>
#   Adam Dybbroe <adam.dybbroe@smhi.se>

# This file is part of mpop.

# mpop is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.

# mpop is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# mpop.  If not, see <http://www.gnu.org/licenses/>.

"""This modules describes the seviri instrument.
"""
import numpy as np

import mpop.imageo.geo_image as geo_image
from mpop.instruments.visir import VisirCompositer
import logging

LOG = logging.getLogger(__name__)

try:
    from pyorbital.astronomy import sun_zenith_angle as sza
except ImportError:
    sza = None


from mpop.imageo import palettes

oca_palette_func = {'ll_ctp': palettes.get_ctp_legend,
                    'ul_ctp': palettes.get_ctp_legend,
                    'ul_cot': palettes.get_cot_legend,
                    'll_cot': palettes.get_cot_legend,
                    'reff': palettes.get_reff_legend,
                    'scenetype': palettes.oca_get_scenetype_legend}


def _arrange_log_data(arr, max_value, no_data_value):
    """
    Prepare logarithmic data for creating an image.

    """
    MAX_IM_VAL = 2**8 - 1

    # Logarithmic data should never be negative
    assert ((arr >= 0) + (arr == no_data_value)).all(), \
        "Negative values encountered in cloud optical thickness"

    # Confine image data values to the range [2, MAX_IM_VAL]
    arr_log = np.log(arr + 1.)  # arr == 0 -> arr_log = 0
    cot_im_data = arr_log * (MAX_IM_VAL - 3) / np.log(max_value + 1.) + 2.
    cot_im_data[cot_im_data > MAX_IM_VAL] = MAX_IM_VAL

    # Now that the data is adjusted, cast it to uint8 ([0, 2**8 - 1])
    cot_im_data = cot_im_data.astype(np.uint8)

    # Give no-data values a special color
    cot_im_data[arr == no_data_value] = 0
    # Give arr == 0 a special color
    cot_im_data[arr == 0] = 1

    return cot_im_data


class SeviriCompositer(VisirCompositer):

    """This class sets up the Seviri instrument channel list.
    """

    instrument_name = "seviri"

    def co2corr(self):
        """CO2 correction of the brightness temperature of the MSG 3.9um
        channel::

        .. math::

          T4_CO2corr = (BT(IR3.9)^4 + Rcorr)^0.25
          Rcorr = BT(IR10.8)^4 - (BT(IR10.8)-dt_CO2)^4
          dt_CO2 = (BT(IR10.8)-BT(IR13.4))/4.0


        """
        try:
            self.check_channels(3.75, 10.8, 13.4)
        except RuntimeError:
            LOG.warning("CO2 correction not performed, channel data missing.")
            return

        bt039 = self[3.9].data
        bt108 = self[10.8].data
        bt134 = self[13.4].data

        dt_co2 = (bt108 - bt134) / 4.0
        rcorr = bt108 ** 4 - (bt108 - dt_co2) ** 4

        t4_co2corr = bt039 ** 4 + rcorr
        t4_co2corr = np.ma.where(t4_co2corr > 0.0, t4_co2corr, 0)
        t4_co2corr = t4_co2corr ** 0.25

        return t4_co2corr

    co2corr.prerequisites = set([3.75, 10.8, 13.4])

    def co2corr_chan(self):
        """CO2 correction of the brightness temperature of the MSG 3.9um
        channel, adding it as a channel::

        .. math::

          T4_CO2corr = (BT(IR3.9)^4 + Rcorr)^0.25
          Rcorr = BT(IR10.8)^4 - (BT(IR10.8)-dt_CO2)^4
          dt_CO2 = (BT(IR10.8)-BT(IR13.4))/4.0


        """

        if "_IR39Corr" in [chn.name for chn in self._data_holder.channels]:
            return

        self.check_channels(3.75, 10.8, 13.4)

        dt_co2 = (self[10.8] - self[13.4]) / 4.0
        rcorr = self[10.8] ** 4 - (self[10.8] - dt_co2) ** 4
        t4_co2corr = self[3.9] ** 4 + rcorr
        t4_co2corr.data = np.ma.where(
            t4_co2corr.data > 0.0, t4_co2corr.data, 0)
        t4_co2corr = t4_co2corr ** 0.25

        t4_co2corr.name = "_IR39Corr"
        t4_co2corr.area = self[3.9].area
        t4_co2corr.wavelength_range = self[3.9].wavelength_range
        t4_co2corr.resolution = self[3.9].resolution

        self._data_holder.channels.append(t4_co2corr)

    co2corr_chan.prerequisites = set([3.75, 10.8, 13.4])

    def refl39_chan(self):
        """Derive the solar (reflectance) part of the 3.9um channel including a
        correction of the limb cooling (co2 correction), adding it as a
        channel.

        """

        if "_IR39Refl" in [chn.name for chn in self._data_holder.channels]:
            return

        if not sza:
            LOG.warning("3.9 reflectance derivation is not possible..." +
                        "\nCheck that pyspectral and pyorbital are " +
                        "installed and available!")
            return

        self.check_channels(3.75, 10.8, 13.4)

        platform_name = self.fullname
        if platform_name == 'unknown':
            LOG.error("Failed setting correct platform name for pyspectral! " +
                      "Satellite = " + str(self.fullname))
        LOG.debug("Satellite = " + str(self.fullname))

        r39 = self[3.9].get_reflectance(self[10.8].data,
                                        sun_zenith=None,
                                        tb13_4=self[13.4].data)

        if r39 is None:
            raise RuntimeError("Couldn't derive 3.x reflectance. " +
                               "Check if pyspectral is installed!")

        r39channel = self[3.9] * 1.0
        r39channel.data = np.ma.where(r39channel.data > 0.0,
                                      r39 * 100, 0)
        r39channel.name = "_IR39Refl"
        r39channel.area = self[3.9].area
        r39channel.wavelength_range = self[3.9].wavelength_range
        r39channel.resolution = self[3.9].resolution

        self._data_holder.channels.append(r39channel)

    refl39_chan.prerequisites = set([3.75, 10.8, 13.4])

    def convection_co2(self):
        """Make a Severe Convection RGB image composite on SEVIRI compensating
        for the CO2 absorption in the 3.9 micron channel.
        """
        self.co2corr_chan()
        self.check_channels("_IR39Corr", 0.635, 1.63, 6.7, 7.3, 10.8)

        ch1 = self[6.7].data - self[7.3].data
        ch2 = self["_IR39Corr"].data - self[10.8].data
        ch3 = self[1.63].check_range() - self[0.635].check_range()

        img = geo_image.GeoImage((ch1, ch2, ch3),
                                 self.area,
                                 self.time_slot,
                                 fill_value=(0, 0, 0),
                                 mode="RGB",
                                 crange=((-30, 0),
                                         (0, 55),
                                         (-70, 20)))

        img.enhance(gamma=(1.0, 0.5, 1.0))

        return img

    convection_co2.prerequisites = (co2corr_chan.prerequisites |
                                    set([0.635, 1.63, 6.7, 7.3, 10.8]))

    def cloudtop(self, stretch=(0.005, 0.005), gamma=None):
        """Make a Cloudtop RGB image composite from Seviri channels.
        """
        self.co2corr_chan()
        self.check_channels("_IR39Corr", 10.8, 12.0)

        ch1 = -self["_IR39Corr"].data
        ch2 = -self[10.8].data
        ch3 = -self[12.0].data

        img = geo_image.GeoImage((ch1, ch2, ch3),
                                 self.area,
                                 self.time_slot,
                                 fill_value=(0, 0, 0),
                                 mode="RGB")

        if stretch:
            img.enhance(stretch=stretch)
        if gamma:
            img.enhance(gamma=gamma)

        return img

    cloudtop.prerequisites = co2corr_chan.prerequisites | set([10.8, 12.0])

    def night_overview(self, stretch='histogram', gamma=None):
        """See cloudtop.
        """
        return self.cloudtop(stretch=stretch, gamma=gamma)

    night_overview.prerequisites = cloudtop.prerequisites

    def night_fog(self):
        """Make a Night Fog RGB image composite from Seviri channels.
        """
        self.co2corr_chan()
        self.check_channels("_IR39Corr", 10.8, 12.0)

        ch1 = self[12.0].data - self[10.8].data
        ch2 = self[10.8].data - self["_IR39Corr"].data
        ch3 = self[10.8].data

        img = geo_image.GeoImage((ch1, ch2, ch3),
                                 self.area,
                                 self.time_slot,
                                 fill_value=(0, 0, 0),
                                 mode="RGB",
                                 crange=((-4, 2),
                                         (0, 6),
                                         (243, 293)))

        img.enhance(gamma=(1.0, 2.0, 1.0))

        return img

    night_fog.prerequisites = co2corr_chan.prerequisites | set([10.8, 12.0])

    def night_microphysics(self):
        """Make a Night Microphysics RGB image composite from Seviri channels.
        This is a Eumetsat variant of night_fog.
        See e.g http://oiswww.eumetsat.int/~idds/html/doc/best_practices.pdf
        """
        self.check_channels(3.9, 10.8, 12.0)

        ch1 = self[12.0].data - self[10.8].data
        ch2 = self[10.8].data - self[3.9].data
        ch3 = self[10.8].data

        img = geo_image.GeoImage((ch1, ch2, ch3),
                                 self.area,
                                 self.time_slot,
                                 fill_value=(0, 0, 0),
                                 mode="RGB",
                                 crange=((-4, 2),
                                         (0, 10),
                                         (243, 293)))

        return img

    night_microphysics.prerequisites = set([3.9, 10.8, 12.0])

    def snow(self):
        """Make a 'Snow' RGB as suggested in the MSG interpretation guide
        (rgbpart04.ppt). It is kind of special as it requires the derivation of
        the daytime component of the mixed Terrestrial/Solar 3.9 micron
        channel. Furthermore the sun zenith angle is used.
        """

        self.refl39_chan()
        self.check_channels("_IR39Refl", 0.8, 1.63, 3.75)

        # We calculate the sun zenith angle again. Should be reused if already
        # calculated/available...
        # FIXME!
        lonlats = self[3.9].area.get_lonlats()
        sunz = sza(self.time_slot, lonlats[0], lonlats[1])
        sunz = np.ma.masked_outside(sunz, 0.0, 88.0)
        sunzmask = sunz.mask
        sunz = sunz.filled(88.)

        costheta = np.cos(np.deg2rad(sunz))

        red = np.ma.masked_where(sunzmask, self[0.8].data / costheta)
        green = np.ma.masked_where(sunzmask, self[1.6].data / costheta)

        img = geo_image.GeoImage((red, green, self['_IR39Refl'].data),
                                 self.area,
                                 self.time_slot,
                                 crange=((0, 100), (0, 70), (0, 30)),
                                 fill_value=None, mode="RGB")
        img.gamma((1.7, 1.7, 1.7))

        return img

    snow.prerequisites = refl39_chan.prerequisites | set(
        [0.8, 1.63, 3.75])

    def day_microphysics(self, wintertime=False, fill_value=None):
        """Make a 'Day Microphysics' RGB as suggested in the MSG interpretation guide
        (rgbpart04.ppt). It is kind of special as it requires the derivation of
        the daytime component of the mixed Terrestrial/Solar 3.9 micron
        channel. Furthermore the sun zenith angle is used.
        for black backgroup specify: fill_value=(0,0,0)
        """

        self.refl39_chan()
        self.check_channels(0.8, "_IR39Refl", 10.8)

        # We calculate the sun zenith angle again. Should be reused if already
        # calculated/available...
        # FIXME!
        lonlats = self[3.9].area.get_lonlats()
        sunz = sza(self.time_slot, lonlats[0], lonlats[1])
        sunz = np.ma.masked_outside(sunz, 0.0, 88.0)
        sunzmask = sunz.mask
        sunz = sunz.filled(88.)

        costheta = np.cos(np.deg2rad(sunz))

        if wintertime:
            crange = ((0, 100), (0, 25), (203, 323))
        else:
            crange = ((0, 100), (0, 60), (203, 323))

        red = np.ma.masked_where(sunzmask,
                                 self[0.8].data / costheta)
        green = np.ma.masked_where(sunzmask,
                                   self['_IR39Refl'].data)
        blue = np.ma.masked_where(sunzmask, self[10.8].data)
        img = geo_image.GeoImage((red, green, blue),
                                 self.area,
                                 self.time_slot,
                                 crange=crange,
                                 fill_value=fill_value, mode="RGB")
        if wintertime:
            img.gamma((1.0, 1.5, 1.0))
        else:
            img.gamma((1.0, 2.5, 1.0))  # Summertime settings....

        return img

    day_microphysics.prerequisites = refl39_chan.prerequisites | set(
        [0.8, 10.8])

    def oca(self, fieldname):
        """Make an OCA cloud parameter image"""

        palette = oca_palette_func[fieldname]()
        data = getattr(getattr(self['OCA'], fieldname), 'data')
        if fieldname in ['scenetype']:
            data = data.astype('uint8')

        elif fieldname in ['ul_ctp', 'll_ctp']:
            data = (22. - data / 5000.).astype('Int16')

        elif fieldname in ['reff']:
            data = (data * 1000000. + 0.5).astype('uint8')
            data.fill_value = 255

        elif fieldname in ['ul_cot', 'll_cot']:
            data = np.ma.exp(data * np.ma.log(10))
            max_value = palettes.CPP_COLORS['cot'].breaks[-1][0]
            data.fill_value = 255
            no_data = 255  # FIXME!
            data = _arrange_log_data(data.filled(), max_value, no_data)

        else:
            raise NotImplementedError(
                "No imagery for parameter %s implemented yet...", fieldname)

        img = geo_image.GeoImage(data, self.area,
                                 self.time_slot,
                                 fill_value=(0),
                                 mode="P",
                                 palette=palette)
        return img

    oca.prerequisites = set(['OCA'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2010, 2011, 2014.

# SMHI,
# Folkborgsvägen 1,
# Norrköping,
# Sweden

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This file is part of mpop.

# mpop is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.

# mpop is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# mpop.  If not, see <http://www.gnu.org/licenses/>.

"""Watch files coming in a given directory.
"""
import datetime
import glob
import os
from Queue import Empty
from threading import Thread, Condition

import time

import logging

LOG = logging.getLogger(__name__)


ZERO = datetime.timedelta(seconds=0)


class FileWatcher(Thread):

    """Looks for new files, and queues them.
    """

    def __init__(self, filename_template, file_queue, frequency):
        """Looks for new files arriving at the given *frequency*, and queues
        them.
        """
        Thread.__init__(self)
        self.queue = file_queue
        self.template = filename_template
        self.frequency = datetime.timedelta(minutes=frequency)
        self.running = True
        self.cond = Condition()

    def terminate(self):
        """Terminate thread.
        """
        self.running = False
        self.cond.acquire()
        self.cond.notify()
        self.cond.release()
        LOG.debug("Termination request received in FileWatcher")

    def wait(self, secs):
        if self.running:
            self.cond.wait(secs)

    def run(self):
        """Run the file watcher.
        """

        filelist = set()
        sleep_time = 8

        while self.running:
            self.cond.acquire()
            if isinstance(self.template, (list, tuple)):
                new_filelist = []
                for template in self.template:
                    new_filelist += glob.glob(template)
                new_filelist = set(new_filelist)
            else:
                new_filelist = set(glob.glob(self.template))
            files_to_process = list(new_filelist - filelist)
            filelist = new_filelist

            files_dict = {}
            for fil in files_to_process:
                files_dict[fil] = os.path.getmtime(fil)

            files_to_process.sort(lambda x, y: cmp(files_dict[x],
                                                   files_dict[y]))

            if len(files_to_process) != 0 and self.running:
                sleep_time = 8
                times = []
                for i in files_to_process:
                    LOG.debug("queueing %s..." % i)
                    self.queue.put(i)
                    times.append(os.stat(i).st_ctime)
                times.sort()

                since_creation = datetime.timedelta(seconds=time.time() -
                                                    times[-1])
                if(self.frequency > since_creation):
                    to_wait = self.frequency - since_creation

                    LOG.info(
                        "Waiting at least " + str(to_wait) + " for next file")
                    sleep_time = (to_wait.seconds +
                                  to_wait.microseconds / 1000000.0)
                    self.wait(sleep_time)
                    sleep_time = 8
            elif self.running:
                LOG.info("no new file has come, waiting %s secs"
                         % str(sleep_time))
                self.wait(sleep_time)
                if sleep_time < 60:
                    sleep_time *= 2

            self.cond.release()
        LOG.info("FileWatcher terminated.")


class FileProcessor(Thread):

    """Execute *fun* on filenames provided by from *file_queue*. If *refresh*
    is a positive number, run *fun* every given number of seconds with None as
    argument.
    """

    def __init__(self, file_queue, fun, refresh=None):
        Thread.__init__(self)
        self.queue = file_queue
        self.fun = fun
        self.running = True
        self.refresh = refresh

    def terminate(self):
        """Terminate thread.
        """
        self.running = False
        LOG.debug("Termination request received in FileProcessor")

    def run(self):
        """Execute the given function on files from the file queue.
        """

        while self.running:
            try:
                filename = self.queue.get(block=True, timeout=self.refresh)
                LOG.debug("processing %s" % filename)
            except Empty:
                filename = None
            try:
                self.fun(filename)
            except:
                LOG.exception("Something wrong happened in %s for %s. Skipping."
                              % (str(self.fun), filename))
        LOG.info("FileProcessor terminated.")

    def stop(self):
        """Stops a running process.
        """
        self.running = False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016 Adam.Dybbroe

# Author(s):

#   Adam.Dybbroe <a000680@c20671.ad.smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""A reader for the EUMETSAT MPEF OCA cloud parameters. Data are segmented LRIT
encoded Grib messages

"""

import tempfile
import os.path
from ConfigParser import ConfigParser
from mpop.satin.gribformat import Grib
import mpop.channel
from mpop import CONFIG_PATH
from mpop.plugin_base import Reader
from trollsift import parser
from glob import glob
import pyresample as pr
import numpy as np

import logging
LOG = logging.getLogger(__name__)

CFG_DIR = os.environ.get('PPP_CONFIG_DIR', './')
AREA_DEF_FILE = os.path.join(CFG_DIR, "areas.def")
if not os.path.exists(AREA_DEF_FILE):
    raise IOError('Config file %s does not exist!' % AREA_DEF_FILE)


LRIT_PATTERN = "L-000-{platform_name:_<5s}_-MPEF________-OCAE_____-{segment:_<9s}-{nominal_time:%Y%m%d%H%M}-{compressed:_<2s}"


SCENE_TYPE_LAYERS = {111: 'Single Layer Water Cloud',
                     112: 'Single Layer Ice Cloud',
                     113: 'Multi Layer Cloud'}

OCA_FIELDS = [{'Pixel scene type': 'Scene type'},
              {'24': 'Measurement Cost',
               'abbrev': 'JM', 'units': ''},
              {'25': 'Upper Layer Cloud Optical Thickness', 'units': '',
               'abbrev': 'ULCOT'},
              {'26': 'Upper Layer Cloud Top Pressure', 'units': 'Pa',
               'abbrev': 'ULCTP'},
              {'27': 'Upper Layer Cloud Effective Radius', 'units': 'm',
               'abbrev': 'ULCRE'},
              {'28': 'Error in Upper Layer Cloud Optical Thickness', 'units': '',
               'abbrev': 'ERR-ULCOT'},
              {'29': 'Error in Upper Layer Cloud Top Pressure', 'units': 'Pa',
               'abbrev': 'ERR-ULCTP'},
              {'30': 'Error in Upper Layer Cloud Effective Radius', 'units': 'm',
               'abbrev': 'ERR-ULCRE'},
              {'31': 'Lower Layer Cloud Optical Thickness',
                  'units': '', 'abbrev': 'LLCOT'},
              {'32': 'Lower Layer Cloud Top Pressure',
                  'units': 'Pa', 'abbrev': 'LLCTP'},
              {'33': 'Error in Lower Layer Cloud Optical Thickness',
                  'units': '', 'abbrev': 'ERR-LLCOT'},
              {'34': 'Error in Lower Layer Cloud Top Pressure', 'units': 'Pa',
               'abbrev': 'ERR-LLCTP'}]

FIELDNAMES = {'scenetype': ('Pixel scene type', None),
              'cost': ('24', None),
              'ul_cot': ('25', '28'),
              'ul_ctp': ('26', '29'),
              'reff': ('27', '30'),
              'll_cot': ('31', '33'),
              'll_ctp': ('32', '34')}


class OCAField(object):

    """One OCA data field with metadata"""

    def __init__(self, units=None, long_name='', standard_name=''):
        self.units = units
        self.data = None
        self.error = None
        self.long_name = long_name
        self.standard_name = standard_name
        self.info = {}


class OCAData(mpop.channel.GenericChannel):

    """The OCA scene data"""

    def __init__(self):
        mpop.channel.GenericChannel.__init__(self)
        self.name = "OCA"
        self.mda = {}
        self._keys = []
        self._refs = {}

        self._lritfiles = None
        self._gribfilename = None
        self._store_grib = False

        self.resolution = 3000

        self.scenetype = OCAField()
        self.cost = OCAField()
        self.ul_cot = OCAField()
        self.ll_cot = OCAField()
        self.ul_ctp = OCAField()
        self.ll_ctp = OCAField()
        self.reff = OCAField()
        self._projectables = []
        for field in FIELDNAMES.keys():
            self._projectables.append(field)

        self.timeslot = None
        self.area_def = pr.utils.load_area(AREA_DEF_FILE, 'met09globeFull')
        self.shape = None

    def readgrib(self):
        """Read the data"""

        oca = Grib(self._gribfilename)
        self.scenetype.data = oca.get('Pixel scene type')[::-1, ::-1]
        self.scenetype.long_name = OCA_FIELDS[0]['Pixel scene type']

        for field in FIELDNAMES.keys():

            setattr(getattr(self, field), 'data', oca.get(
                FIELDNAMES[field][0])[::-1, ::-1])
            param = [s for s in OCA_FIELDS if FIELDNAMES[field][0] in s][0]
            if 'units' in param:
                setattr(getattr(self, field), 'units', param['units'])
            if 'abbrev' in param:
                setattr(getattr(self, field), 'standard_name', param['abbrev'])
            setattr(getattr(self, field), 'long_name',
                    param[FIELDNAMES[field][0]])
            param_name = FIELDNAMES[field][1]
            if param_name:
                setattr(
                    getattr(self, field), 'error', oca.get(param_name)[::-1, ::-1])

        if not self._store_grib:
            os.remove(self._gribfilename)

    def read_from_lrit(self, filenames, gribfilename=None):
        """Read and concatenate the LRIT segments"""

        self._lritfiles = filenames

        if len(filenames) == 0:
            print("No files provided!")
            return

        if gribfilename:
            self._store_grib = True
            self._gribfilename = gribfilename
        else:
            self._store_grib = False
            self._gribfilename = tempfile.mktemp(suffix='.grb')

        p__ = parser.Parser(LRIT_PATTERN)

        bstr = {}
        nsegments = 0
        for lritfile in self._lritfiles:
            if os.path.basename(lritfile).find('PRO') > 0:
                print("PRO file... %s: Skip it..." % lritfile)
                continue

            res = p__.parse(os.path.basename(lritfile))
            segm = int(res['segment'].strip('_'))
            if not self.timeslot:
                self.timeslot = res['nominal_time']
            LOG.debug("Segment = %d", segm)
            nsegments = nsegments + 1

            with open(lritfile) as fpt:
                fpt.seek(103)
                bstr[segm] = fpt.read()

        fstr = bstr[1]
        for idx in range(2, nsegments + 1):
            fstr = fstr + bstr[idx]

        with open(self._gribfilename, 'wb') as fpt:
            fpt.write(fstr)

        self.readgrib()

    def project(self, coverage):
        """Project the data"""
        LOG.debug("Projecting channel %s...", (self.name))
        import copy
        res = copy.copy(self)

        res.name = self.name
        res.resolution = self.resolution
        res.filled = True
        res.area = coverage.out_area
        resolution_str_x = str(int(res.area.pixel_size_x)) + 'm'
        resolution_str_y = str(int(res.area.pixel_size_y)) + 'm'

        time_axis = 0

        # Project the data
        for var in self._projectables:
            LOG.info("Projecting " + str(var))
            res.__dict__[var] = copy.copy(self.__dict__[var])
            data = coverage.project_array(self.__dict__[var].data)
            valid_min = np.min(data)
            valid_max = np.max(data)

            res.__dict__[var].data = data
            res.__dict__[var].info['var_name'] = var
            res.__dict__[var].info[
                'var_data'] = np.ma.expand_dims(data, time_axis)

            dim_names = ['y' + resolution_str_y,
                         'x' + resolution_str_x]
            dim_names.insert(0, 'time')
            res.__dict__[var].info['var_dim_names'] = dim_names
            res.__dict__[var].info['long_name'] = res.__dict__[var].long_name
            res.__dict__[var].info[
                'standard_name'] = res.__dict__[var].standard_name
            res.__dict__[var].info['valid_range'] = np.array(
                [valid_min, valid_max])
            # res.__dict__[var].info['resolution'] = res.resolution

        return res

    def is_loaded(self):
        """Tells if the channel contains loaded data.
        """
        return True


class OCAReader(Reader):

    pformat = "mpef_oca"

    def load(self, satscene, *args, **kwargs):
        """Read data from files and load it into *satscene*.
        """
        lonlat_is_loaded = False

        lritfiles = kwargs.get('filenames')

        if "OCA" not in satscene.channels_to_load:
            LOG.warning("No OCA product requested. Nothing to be done...")
            return

        area_name = satscene.area_id or satscene.area.area_id
        # platform_name = satscene.satname

        conf = ConfigParser()
        conf.read(os.path.join(CONFIG_PATH, satscene.fullname + ".cfg"))

        # Reading the products
        product = "oca"
        classes = {product: OCAData}

        LOG.debug("Loading " + product)

        if not lritfiles:
            dummy, lritfiles = get_lrit_filenames(satscene, area_name)

        LOG.info("Filenames = " + str(lritfiles))

        chn = classes[product]()
        chn.read_from_lrit(lritfiles)

        # Prepare info object for netCDF writer:
        resolution_str = str(int(chn.resolution)) + 'm'
        for field in chn._projectables:

            getattr(chn, field).info['var_name'] = field
            getattr(chn, field).info['var_data'] = getattr(chn, field).data
            getattr(chn, field).info[
                'var_dir_names'] = getattr(chn, field).data

            getattr(chn, field).info['var_dim_names'] = ('y' + resolution_str,
                                                         'x' + resolution_str)
            getattr(chn, field).info['long_name'] = getattr(
                chn, field).long_name
            getattr(chn, field).info['standard_name'] = getattr(
                chn, field).standard_name
            valid_min = np.min(getattr(chn, field).data)
            valid_max = np.max(getattr(chn, field).data)
            getattr(chn, field).info['valid_range'] = np.array(
                [valid_min, valid_max])
            getattr(chn, field).info['resolution'] = chn.resolution

        satscene.channels.append(chn)

        LOG.info("Loading MPEF OCA cloud parameters done")

        return


def get_lrit_filenames(scene, area_name):
    """Get the set of lrit filenames for the given scene
    """

    conf = ConfigParser()
    conf.read(os.path.join(CONFIG_PATH, scene.fullname + ".cfg"))

    filename = conf.get(scene.instrument_name + "-level4",
                        "filename",
                        raw=True,
                        vars=os.environ)
    directory = conf.get(scene.instrument_name + "-level4",
                         "dir",
                         vars=os.environ)
    pathname_tmpl = os.path.join(directory, filename)
    LOG.debug("Path = " + str(pathname_tmpl))

    fparser = parser.Parser(pathname_tmpl)

    lrit_files = glob(
        parser.globify(pathname_tmpl, {'nominal_time': scene.time_slot}))

    prologue = None
    segmfiles = []
    segm_numbers = []
    for item in lrit_files:
        p__ = fparser.parse(item)
        segm = p__['segment'].strip('_')
        if segm == 'PRO':
            prologue = item
        else:
            segm_numbers.append(int(segm))
            segmfiles.append(item)

    if not prologue:
        LOG.warning("No prologue file found for timeslot")

    segm_numbers.sort()
    if range(1, 11) == segm_numbers:
        LOG.info("All ten segment files found")
    else:
        LOG.warning("Less than 10 segments found: %s", str(segm_numbers))

    return prologue, segmfiles



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2009, 2012.

# SMHI,
# Folkborgsvägen 1,
# Norrköping,
# Sweden

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>
#   Adam Dybbroe <adam.dybbroe@smhi.se>
#   Esben S. Nielsen <esn@dmi.dk>

# This file is part of mpop.

# mpop is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# mpop is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with mpop.  If not, see <http://www.gnu.org/licenses/>.

"""Module defining various utilities.
"""

import os
import re
import ConfigParser
import logging

from mpop import CONFIG_PATH


class OrderedConfigParser(object):

    """Intercepts read and stores ordered section names.
    Cannot use inheritance and super as ConfigParser use old style classes.
    """

    def __init__(self, *args, **kwargs):
        self.config_parser = ConfigParser.ConfigParser(*args, **kwargs)

    def __getattr__(self, name):
        return getattr(self.config_parser, name)

    def read(self, filename):
        """Reads config file
        """

        try:
            conf_file = open(filename, 'r')
            config = conf_file.read()
            config_keys = re.findall(r'\[.*\]', config)
            self.section_keys = [key[1:-1] for key in config_keys]
        except IOError, e:
            # Pass if file not found
            if e.errno != 2:
                raise

        return self.config_parser.read(filename)

    def sections(self):
        """Get sections from config file
        """

        try:
            return self.section_keys
        except:
            return self.config_parser.sections()


def ensure_dir(filename):
    """Checks if the dir of f exists, otherwise create it.
    """
    import os
    directory = os.path.dirname(filename)
    if len(directory) and not os.path.isdir(directory):
        os.makedirs(directory)


class NullHandler(logging.Handler):

    """Empty handler.
    """

    def emit(self, record):
        """Record a message.
        """
        pass


def debug_on():
    """Turn debugging logging on.
    """
    logging_on(logging.DEBUG)

_is_logging_on = False

# Read default log level from mpop's config file
_config = ConfigParser.ConfigParser()
_config.read(os.path.join(CONFIG_PATH, 'mpop.cfg'))
try:
    default_loglevel = _config.get('general', 'loglevel')
except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
    default_loglevel = 'WARNING'
default_loglevel = getattr(logging, default_loglevel.upper())
del _config

# logging_on(default_loglevel)


def logging_on(level=default_loglevel):
    """Turn logging on.
    """
    global _is_logging_on

    if not _is_logging_on:
        console = logging.StreamHandler()
        console.setFormatter(logging.Formatter("[%(levelname)s: %(asctime)s :"
                                               " %(name)s] %(message)s",
                                               '%Y-%m-%d %H:%M:%S'))
        console.setLevel(level)
        logging.getLogger('').addHandler(console)
        _is_logging_on = True

    log = logging.getLogger('')
    log.setLevel(level)
    for h in log.handlers:
        h.setLevel(level)


def logging_off():
    """Turn logging off.
    """
    logging.getLogger('').handlers = [NullHandler()]


def get_logger(name):
    """Return logger with null handle
    """

    log = logging.getLogger(name)
    if not log.handlers:
        log.addHandler(NullHandler())
    return log


###

import re


def strftime(utctime, format_string):
    """Like datetime.strftime, except it works with string formatting
    conversion specifier items on windows, making the assumption that all
    conversion specifiers use mapping keys.

    E.g.:
    >>> from datetime import datetime
    >>> t = datetime.utcnow()
    >>> a = "blabla%Y%d%m-%H%M%S-%(value)s"
    >>> strftime(t, a)
    'blabla20120911-211448-%(value)s'
    """
    res = format_string
    for i in re.finditer("%\w", format_string):
        res = res.replace(i.group(), utctime.strftime(i.group()))
    return res



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Adam.Dybbroe

# Author(s):

#   Adam.Dybbroe <a000680@c20671.ad.smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""A reader for the FY3 Mersi-1 
"""


import numpy as np
import os
import logging
from datetime import datetime
import glob
from ConfigParser import ConfigParser
from mpop import CONFIG_PATH
import h5py
import pdb

LOGGER = logging.getLogger('mersi-1')


def load(satscene, *args, **kwargs):
    """Read data from file and load it into *satscene*.
    A possible *calibrate* keyword argument is passed to the AAPP reader. 
    Should be 0 for off (counts), 1 for default (brightness temperatures and
    reflectances), and 2 for radiances only.

    If *use_extern_calib* keyword argument is set True, use external
    calibration data.

    """
    del args

    conf = ConfigParser()
    conf.read(os.path.join(CONFIG_PATH, satscene.fullname + ".cfg"))
    options = {}
    for option, value in conf.items(satscene.instrument_name + "-level2",
                                    raw=True):
        options[option] = value

    if kwargs.get("filename") is not None:
        options["full_filename"] = kwargs["filename"]
    if kwargs.get("calibrate") is not None:
        options["calibrate"] = kwargs["calibrate"]
    else:
        options["calibrate"] = True

    LOGGER.info("Loading instrument '%s'", satscene.instrument_name)

    try:
        CASES[satscene.instrument_name](satscene, options)
    except KeyError:
        raise KeyError("Unknown instrument '%s'" % satscene.instrument_name)


def load_mersi(satscene, options):
    """Read the Mersi-1 hdf file"""

    if "filename_1000m" not in options:
        raise IOError("No 1km mersi-1 filename given, cannot load.")

    values = {"orbit": satscene.orbit,
              "satname": satscene.satname,
              "instrument": satscene.instrument_name,
              "satellite": satscene.fullname
              }

    filename_1000m = \
        os.path.join(satscene.time_slot.strftime(options["dir"]) % values,
                     satscene.time_slot.strftime(
                         options["filename_1000m"])
                     % values)

    LOGGER.debug("Filename= %s", filename_1000m)

    datasets = ['EV_250_Aggr.1KM_RefSB',
                'EV_250_Aggr.1KM_Emissive',
                'EV_1KM_RefSB']

    calibrate = options['calibrate']

    # Get the calibration information:
    h5f = h5py.File(filename_1000m)
    # The K0, K1 and K2 coefficients:
    vis_cal_coeff = h5f['Calibration']['VIS_Cal_Coeff'][:]
    # See also "Update of Calibration for Reflective Solar Bands of MERSI / FY-3C"
    # http://satellite.cma.gov.cn/PortalSite/Download/FY3C/CalibrationCoefficient/Update%20of%20Calibration%20for%20Reflective%20Solar%20Bands%20of%20MERSI_20140618.doc

    sv_dn_average = h5f['Calibration']['SV_DN_average'][:]
    # Expand array over all lines (10 lines per scan):
    sv_dn_average = np.repeat(sv_dn_average, 10, axis=1)

    date_orig = h5f.attrs['DN_Normalized_LUT_UpdateDate']
    dtobj_orig = datetime.strptime(date_orig, '%Y-%m-%d')
    obs_beg_date = h5f.attrs["Observing Beginning Date"]
    obs_beg_time = h5f.attrs["Observing Beginning Time"]
    dtobj_obs = datetime.strptime(
        obs_beg_date + obs_beg_time, '%Y-%m-%d%H:%M:%S.%f')
    h5f.close()

    # Get the days since 'launch' or since coefficients update:
    dsl = (dtobj_obs - dtobj_orig).days
    slopes = (vis_cal_coeff[:, 0] +
              vis_cal_coeff[:, 1] * dsl +
              vis_cal_coeff[:, 2] * dsl * dsl)
    # The slopes are available for band 1-4 and 6-20.
    # To keep consistency with the other cal-coefficients we add the IR band as
    # well, and set the slope to 1:
    slopes = np.concatenate((slopes[0:4], [1], slopes[4:]))

    mersi_band_index = 0
    with h5py.File(filename_1000m) as h5f:

        for dset in datasets:
            band_data = h5f['Data'][dset]
            valid_range = band_data.attrs['valid_range']
            LOGGER.debug("valid-range = " + str(valid_range))
            # FIXME! There seem to be useful data outside the valid range!
            valid_range = (0, 65535)
            fillvalue = band_data.attrs['FillValue']
            band_names = band_data.attrs['band_name'].split(',')
            slope = band_data.attrs['Slope']
            intercept = band_data.attrs['Intercept']

            LOGGER.debug('band names = ' + str(band_names))
            for (i, band) in enumerate(band_names):
                if band not in satscene.channels_to_load:
                    continue

                LOGGER.debug("Reading channel %s, i=%d", band, i)

                # Take care of the case when there is only one
                # single band (band 5: IR) in the dataset:
                if len(band_data.shape) == 2:
                    data = band_data
                else:
                    data = band_data[i]

                bandmask = np.logical_or(np.less(data, valid_range[0]),
                                         np.greater(data, valid_range[1]))

                if calibrate:
                    data = slopes[mersi_band_index] * (
                        data - np.array([sv_dn_average[mersi_band_index]]).transpose())

                satscene[band] = np.ma.masked_array(data,
                                                    mask=bandmask,
                                                    copy=False)

                satscene[band].info = {
                    'var_name': 'ch' + str(band),
                    'var_data': satscene[band].data,
                    'var_dim_names': ('x', 'y'),
                    '_FillValue': fillvalue,
                    'standard_name': '',
                    'short_name': band,
                    'scale_factor': slope,
                    'add_offset': intercept,
                }

                mersi_band_index = mersi_band_index + 1

    satscene.info = {
        'Antenna': 'None',
        'Receiver': 'Unknown',
        'Time': satscene.time_slot.strftime("%Y-%m-%d %H:%M:%S UTC"),
        'Area_Name': "swath",
        #'Projection': 'satproj',
        'Platform Name': satscene.fullname,
        'Service': '',
        #'Columns' : satscene.channels[0].shape[1],
        #'Lines' : satscene.channels[0].shape[0],
        'SampleX': 1.0,
        'SampleY': 1.0,
        'title': 'MERSI Level 1',
    }

    # Get geolocation information


CASES = {
    "mersi/1": load_mersi,
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Adam.Dybbroe

# Author(s):

#   Adam.Dybbroe <a000680@c20671.ad.smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Unit testing the pps level-2 netCDF reader
"""
import unittest
from mpop.satin.nc_pps_l2 import PPSReader

CTTH_TESTFILE_LOCAL_1 = '/path/to/my/products/S_NWC_CTTH_noaa19_33897_20150906T1240015Z_20150906T1240598Z.nc'
CT_TESTFILE_LOCAL_1 = '/path/to/my/products/S_NWC_CT_noaa19_33897_20150906T1240015Z_20150906T1240598Z.nc'
CPP_TESTFILE_LOCAL_1 = '/path/to/my/products/S_NWC_CPP_noaa19_33897_20150906T1240015Z_20150906T1240598Z.nc'
CMA_TESTFILE_LOCAL_1 = '/path/to/my/products/S_NWC_CMA_noaa19_33897_20150906T1240015Z_20150906T1240598Z.nc'
PC_TESTFILE_LOCAL_1 = '/path/to/my/products/S_NWC_PC_noaa19_33897_20150906T1240015Z_20150906T1240598Z.nc'
GEO_TESTFILE_LOCAL_1 = '/path/to/my/products/S_NWC_CMA_noaa19_33897_20150906T1240015Z_20150906T1240598Z.nc'


CTTH_TESTFILE_EARS_1 = '/path/to/my/products/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,METOPB+CTTH_C_EUMS_20150914093100_15510.nc.bz2'
CT_TESTFILE_EARS_1 = '/path/to/my/products/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,METOPB+CT_C_EUMS_20150914093100_15510.nc.bz2'
CMA_TESTFILE_EARS_1 = '/path/to/my/products/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,METOPB+CMA_C_EUMS_20150914093100_15510.nc.bz2'
CTTH_TESTFILE_EARS_2 = '/path/to/my/products/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,METOPB+CTTH_C_EUMS_20150914093200_15510.nc.bz2'
CT_TESTFILE_EARS_2 = '/path/to/my/products/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,METOPB+CT_C_EUMS_20150914093200_15510.nc.bz2'
CMA_TESTFILE_EARS_2 = '/path/to/my/products/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,METOPB+CMA_C_EUMS_20150914093200_15510.nc.bz2'


class TestPPSReader(unittest.TestCase):

    """Class for testing the PPSReader reader class.
    """

    def setUp(self):
        self.reader = PPSReader(object)

    def test_determine_prod_and_geo_files_local(self):
        """Test the private method _determine_prod_and_geo_files """
        self.reader._source = 'local'
        self.reader._cloud_product_geodir = None
        self.reader._geolocation_product_name = 'CMA'
        pro, geo = self.reader._determine_prod_and_geo_files(
            CTTH_TESTFILE_LOCAL_1)
        self.assertEqual(pro.keys(), ['CTTH'])
        self.assertEqual(pro['CTTH'], [CTTH_TESTFILE_LOCAL_1])
        self.assertEqual(geo.keys(), ['CTTH'])
        self.assertEqual(geo['CTTH'], [GEO_TESTFILE_LOCAL_1])

        pro, geo = self.reader._determine_prod_and_geo_files(
            [CTTH_TESTFILE_LOCAL_1, CT_TESTFILE_LOCAL_1, CMA_TESTFILE_LOCAL_1])
        self.assertEqual(len(pro.keys()), 3)
        self.assertTrue('CMA' in pro.keys())
        self.assertTrue('CT' in geo.keys())
        self.assertTrue('CTTH' in geo.keys())
        self.assertEqual(geo['CTTH'], geo['CT'])
        self.assertEqual(geo['CTTH'], geo['CMA'])

    def test_determine_prod_and_geo_files_ears(self):
        """Test the private method _determine_prod_and_geo_files """
        self.reader._source = 'ears'
        self.reader._cloud_product_geodir = None
        self.reader._geolocation_product_name = None
        pro, geo = self.reader._determine_prod_and_geo_files(
            CTTH_TESTFILE_EARS_1)
        self.assertEqual(pro.keys(), ['CTTH'])
        self.assertEqual(pro['CTTH'], [CTTH_TESTFILE_EARS_1])

        pro, geo = self.reader._determine_prod_and_geo_files(
            [CTTH_TESTFILE_EARS_1, CT_TESTFILE_EARS_1, CMA_TESTFILE_EARS_1])
        nkeys = len(pro.keys())
        self.assertEqual(nkeys, 3)
        for key in pro.keys():
            self.assertTrue(key in ["CTTH", "CT", "CMA"])

        for key in pro.keys():
            self.assertEqual(geo[key], pro[key])

        for key in pro.keys():
            if key not in ['CMA']:
                self.assertNotEqual(geo[key], geo['CMA'])

    def tearDown(self):
        pass


def suite():
    """The test suite for test_viirs_sdr.
    """
    loader = unittest.TestLoader()
    mysuite = unittest.TestSuite()
    mysuite.addTest(loader.loadTestsFromTestCase(TestPPSReader))

    return mysuite
